@@ -51,6 +51,32 @@ module grafic_io
      module procedure grafic_read_single, grafic_read_double
   end interface
 
+  interface
+     subroutine f77_parallel_read &
+          (filename, fnamelen, size, offset, buffer_address)
+       use iso_C_binding
+       implicit none
+       character(len=128) :: filename
+       integer(c_int32_t) :: fnamelen
+       integer(c_int64_t) :: size
+       integer(c_int64_t) :: offset
+       type(c_ptr), value :: buffer_address
+     end subroutine f77_parallel_read
+  end interface
+
+  interface
+     subroutine f77_parallel_write &
+          (filename, fnamelen, size, offset, buffer_address)
+       use iso_C_binding
+       implicit none
+       character(len=128) :: filename
+       integer(c_int32_t) :: fnamelen
+       integer(c_int64_t) :: size
+       integer(c_int64_t) :: offset
+       type(c_ptr), value :: buffer_address
+     end subroutine f77_parallel_write
+  end interface
+
 contains
 
 
@@ -60,6 +86,8 @@ contains
   ! Padded region will not be written.
   subroutine grafic_write_double(buffer,local_nz,local_z_start,ny,nx,filename,padding_in, white_in)
 
+    use iso_C_binding
+
     ! Arguments
     implicit none
     include 'mpif.h'
@@ -71,9 +99,13 @@ contains
     logical, optional :: white_in
 
     ! Local variables
-    real(sp), allocatable, dimension(:) :: tampon
+    !real(sp), allocatable, dimension(:) :: tampon
     integer :: record_size
-    integer(RECORD_FLAG) :: taille_tampon
+    !integer(RECORD_FLAG) :: taille_tampon ! kind values are implementation dependent!
+
+    real(c_float), allocatable, dimension(:), target :: tampon
+    integer(c_int32_t), target :: taille_tampon
+
     integer :: n2x
     integer(i8b) :: index, index2, offset, length, toto
     integer :: idummy=0
@@ -103,9 +135,9 @@ contains
     endif
 
     if (padding) then
-       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG, i8b)
     else
-       offset = offset + local_z_start*int(ny*nx*sp_bytes + 2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*nx*sp_bytes + 2*RECORD_FLAG, i8b)
     endif
 
     do k=1,local_nz
@@ -131,21 +163,21 @@ contains
        endif
        ! First write f... Fortran record length
        length=RECORD_FLAG
-       call f77_parallel_write(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_write(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        offset = offset+RECORD_FLAG
        if (padding) then
           length = ny*n2x*sp_bytes
        else
           length = ny*nx*sp_bytes
        endif
-       call f77_parallel_write(trim(filename),len_trim(filename), length, &
-            & offset, tampon)
+       call f77_parallel_write(trim(filename),len_trim(filename, i4b), length, &
+            & offset, c_loc(tampon))
        offset = offset+length
        ! Again ..
        length=RECORD_FLAG
-       call f77_parallel_write(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_write(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        offset = offset+RECORD_FLAG
 
     enddo
@@ -154,6 +186,7 @@ contains
   end subroutine grafic_write_double
 
   subroutine grafic_write_single(buffer,local_nz,local_z_start,ny,nx,filename,padding_in, white_in)
+    use iso_C_binding
 
     ! Arguments
     implicit none
@@ -167,9 +200,13 @@ contains
 
 
     ! Local variables
-    real(sp), allocatable, dimension(:) :: tampon
+    !real(sp), allocatable, dimension(:) :: tampon
     integer :: record_size
-    integer(RECORD_FLAG) :: taille_tampon
+    !integer(RECORD_FLAG) :: taille_tampon
+
+    real(c_float), allocatable, dimension(:), target :: tampon
+    integer(c_int32_t), target :: taille_tampon
+
     integer :: n2x
     integer(i8b) :: index, index2, offset, length, toto
     integer :: idummy=0
@@ -200,9 +237,9 @@ contains
     endif
 
     if (padding) then
-       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG, i8b)
     else
-       offset = offset + local_z_start*int(ny*nx*sp_bytes + 2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*nx*sp_bytes + 2*RECORD_FLAG, i8b)
     endif
 
     do k=1,local_nz
@@ -228,21 +265,21 @@ contains
        endif
        ! First write f... Fortran record length
        length=RECORD_FLAG
-       call f77_parallel_write(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_write(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        offset = offset+RECORD_FLAG
        if (padding) then
           length = ny*n2x*sp_bytes
        else
           length = ny*nx*sp_bytes
        endif
-       call f77_parallel_write(trim(filename),len_trim(filename), length, &
-            & offset, tampon)
+       call f77_parallel_write(trim(filename),len_trim(filename, i4b), length, &
+            & offset, c_loc(tampon))
        offset = offset+length
        ! Again ..
        length=RECORD_FLAG
-       call f77_parallel_write(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_write(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        offset = offset+RECORD_FLAG
 
     enddo
@@ -252,6 +289,8 @@ contains
 
   subroutine grafic_read_double(buffer,local_nz,local_z_start,ny,nx,filename, padding_in, white_in)
 
+    use iso_C_binding
+
     ! Arguments
     implicit none
     include 'mpif.h'
@@ -261,9 +300,13 @@ contains
     logical, optional :: padding_in ! Read padded zone or not
     logical, optional :: white_in ! Different header for white noise file
     ! Local variables
-    real(sp), allocatable, dimension(:) :: tampon
+    !real(sp), allocatable, dimension(:) :: tampon
     integer :: record_size
-    integer(RECORD_FLAG) :: taille_tampon
+    !integer(RECORD_FLAG) :: taille_tampon ! kind values are implementation dependent!
+
+    real(c_float), allocatable, dimension(:), target :: tampon
+    integer(c_int32_t), target :: taille_tampon
+
     integer :: n2x
     integer(i8b) :: index, index2, offset, length
     integer :: idummy=0
@@ -292,17 +335,17 @@ contains
        offset = 3*4+8*sp_bytes + 2*RECORD_FLAG ! First record
     endif
     if (padding) then
-       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG, i8b)
     else
-       offset = offset + local_z_start*int(ny*nx*sp_bytes+2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*nx*sp_bytes+2*RECORD_FLAG, i8b)
     endif
 
     do k=1,local_nz
 
        length = RECORD_FLAG
        ! First read f... Fortran record length
-       call f77_parallel_read(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_read(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        if (padding) then
            if (taille_tampon /= ny*n2x*sp_bytes) then
              print*,'Record size ',taille_tampon,' is different from expected', &
@@ -323,13 +366,13 @@ contains
        else
           length = ny*nx*sp_bytes
        endif
-       call f77_parallel_read(trim(filename),len_trim(filename), length, &
-            & offset, tampon)
+       call f77_parallel_read(trim(filename),len_trim(filename, i4b), length, &
+            & offset, c_loc(tampon))
        offset = offset+length
        ! Again ..
        length=RECORD_FLAG
-       call f77_parallel_read(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_read(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        offset = offset+RECORD_FLAG
 
        ! This loop to (eventually) get rid of fftw padding zone
@@ -358,6 +401,7 @@ contains
   end subroutine grafic_read_double
 
   subroutine grafic_read_single(buffer,local_nz,local_z_start,ny,nx,filename, padding_in, white_in)
+    use iso_C_binding
 
     ! Arguments
     implicit none
@@ -368,9 +412,13 @@ contains
     logical, optional :: padding_in ! Read padded zone or not
     logical, optional :: white_in ! Different header for white noise file
     ! Local variables
-    real(sp), allocatable, dimension(:) :: tampon
+    !real(sp), allocatable, dimension(:) :: tampon
     integer :: record_size
-    integer(RECORD_FLAG) :: taille_tampon
+    !integer(RECORD_FLAG) :: taille_tampon
+
+    real(c_float), allocatable, dimension(:), target :: tampon
+    integer(c_int32_t), target :: taille_tampon
+
     integer :: n2x
     integer(i8b) :: index, index2, offset, length
     integer :: idummy=0
@@ -399,17 +447,17 @@ contains
        offset = 3*4+8*sp_bytes + 2*RECORD_FLAG ! First record
     endif
     if (padding) then
-       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*n2x*sp_bytes + 2*RECORD_FLAG, i8b)
     else
-       offset = offset + local_z_start*int(ny*nx*sp_bytes+2*RECORD_FLAG,8)
+       offset = offset + local_z_start*int(ny*nx*sp_bytes+2*RECORD_FLAG, i8b)
     endif
 
     do k=1,local_nz
 
        length = RECORD_FLAG
        ! First read f... Fortran record length
-       call f77_parallel_read(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_read(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        if (padding) then
            if (taille_tampon /= ny*n2x*sp_bytes) then
              print*,'Record size ',taille_tampon,' is different from expected', &
@@ -430,13 +478,13 @@ contains
        else
           length = ny*nx*sp_bytes
        endif
-       call f77_parallel_read(trim(filename),len_trim(filename), length, &
-            & offset, tampon)
+       call f77_parallel_read(trim(filename),len_trim(filename, i4b), length, &
+            & offset, c_loc(tampon))
        offset = offset+length
        ! Again ..
        length=RECORD_FLAG
-       call f77_parallel_read(trim(filename),len_trim(filename), length &
-            & ,offset,taille_tampon)
+       call f77_parallel_read(trim(filename),len_trim(filename, i4b), length &
+            & ,offset,c_loc(taille_tampon))
        offset = offset+RECORD_FLAG
 
        ! This loop to (eventually) get rid of fftw padding zone
